@@ -40,6 +40,13 @@
 
 #include "gmconfwidgets.h"
 
+#include <string.h>
+
+enum {
+  COLUMN_STRING_RAW = 0, /* must be zero because it's used in gmconfwidgets */
+  COLUMN_STRING_TRANSLATED,
+  COLUMN_SENSITIVE,
+};
 
 static void tree_selection_changed_cb (GtkTreeSelection *,
 				       gpointer);
@@ -51,7 +58,6 @@ typedef struct _GnomePrefsWindow {
   GtkWidget *section_label;
   GtkWidget *sections_tree_view;
   GtkTreeIter iter;
-  GtkTooltips *tips;
   int last_page;
   
 } GnomePrefsWindow;
@@ -106,7 +112,7 @@ gnome_prefs_entry_new (GtkWidget *table,
 		       gboolean box)
 {
   GnomePrefsWindow *gpw = NULL;
-  GValue value = { 0 };
+  GValue value = { 0, {{0}, {0}}};
   int cols = 0;
   GtkWidget *entry = NULL;
   GtkWidget *label = NULL;
@@ -165,10 +171,12 @@ gnome_prefs_entry_new (GtkWidget *table,
   g_free (conf_string);
 
   g_signal_connect_after (G_OBJECT (entry), "focus-out-event",
-			  G_CALLBACK (entry_focus_changed), (gpointer) conf_key);
+			  G_CALLBACK (entry_focus_changed),
+			  (gpointer)conf_key);
 
   g_signal_connect_after (G_OBJECT (entry), "activate",
-			  G_CALLBACK (entry_activate_changed), (gpointer) conf_key);
+			  G_CALLBACK (entry_activate_changed),
+			  (gpointer)conf_key);
 
   gm_conf_notifier_add (conf_key, entry_changed_nt, (gpointer) entry);
 
@@ -180,7 +188,7 @@ gnome_prefs_entry_new (GtkWidget *table,
 
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && tooltip)
-    gtk_tooltips_set_tip (gpw->tips, entry, tooltip, NULL);
+    gtk_widget_set_tooltip_text (entry, tooltip);
 
   gtk_widget_show_all (table);
   
@@ -196,7 +204,7 @@ gnome_prefs_toggle_new (GtkWidget *table,
 			int row)
 {
   GnomePrefsWindow *gpw = NULL;
-  GValue value = { 0 };
+  GValue value = { 0, {{0}, {0}}};
   GtkWidget *toggle = NULL;
   gboolean writable = FALSE;
   int cols = 0;
@@ -222,7 +230,7 @@ gnome_prefs_toggle_new (GtkWidget *table,
 
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && tooltip)
-    gtk_tooltips_set_tip (gpw->tips, toggle, tooltip, NULL);
+    gtk_widget_set_tooltip_text (toggle, tooltip);
 
   g_signal_connect (G_OBJECT (toggle), "toggled",
 		    G_CALLBACK (toggle_changed), (gpointer) conf_key);
@@ -299,7 +307,7 @@ gnome_prefs_scale_new (GtkWidget *table,
 
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && tooltip)
-    gtk_tooltips_set_tip (gpw->tips, hscale, tooltip, NULL);
+    gtk_widget_set_tooltip_text (hscale, tooltip);
 
   g_signal_connect (G_OBJECT (adj), "value-changed",
 		    G_CALLBACK (adjustment_changed),
@@ -359,7 +367,7 @@ gnome_prefs_spin_new (GtkWidget *table,
   adj = (GtkAdjustment *) 
     gtk_adjustment_new (gm_conf_get_int (conf_key),
 			min, max, step,
-			2.0, 1.0);
+			10.0, 1.0);
   
   spin_button = gtk_spin_button_new (adj, 1.0, 0);
   if (!writable)
@@ -395,7 +403,7 @@ gnome_prefs_spin_new (GtkWidget *table,
 
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && tooltip)
-    gtk_tooltips_set_tip (gpw->tips, spin_button, tooltip, NULL);
+    gtk_widget_set_tooltip_text (spin_button, tooltip);
 
   g_signal_connect (G_OBJECT (adj), "value-changed",
 		    G_CALLBACK (adjustment_changed),
@@ -487,8 +495,8 @@ gnome_prefs_range_new (GtkWidget *table,
 
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && spin1_tooltip && spin2_tooltip) {
-    gtk_tooltips_set_tip (gpw->tips, spin_button1, spin1_tooltip, NULL);
-    gtk_tooltips_set_tip (gpw->tips, spin_button2, spin2_tooltip, NULL);
+    gtk_widget_set_tooltip_text (spin_button1, spin1_tooltip);
+    gtk_widget_set_tooltip_text (spin_button2, spin2_tooltip);
   }
   
 
@@ -521,67 +529,79 @@ gnome_prefs_int_option_menu_new (GtkWidget *table,
 				 int row)
 {
   GnomePrefsWindow *gpw = NULL;
-  GtkWidget *item = NULL;
-  GtkWidget *label = NULL;
+  
+  GtkWidget *label = NULL;                                                     
   GtkWidget *option_menu = NULL;
-  GtkWidget *menu = NULL;
-
-  int cpt = 0;
+  
+  GtkListStore *list_store = NULL;
+  GtkCellRenderer *renderer = NULL;
+  GtkTreeIter iter;
+  
   gboolean writable = FALSE;
-  
+
+  int history = -1;
+  int cpt = 0;
+
   writable = gm_conf_is_key_writable (conf_key);
-  
+
   label = gtk_label_new_with_mnemonic (label_txt);
   if (!writable)
     gtk_widget_set_sensitive (GTK_WIDGET (label), FALSE);
   
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,                
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL),
                     0, 0);
-
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+                                                                               
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);                         
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
-  menu = gtk_menu_new ();
-  option_menu = gtk_option_menu_new ();
+  list_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+  option_menu = gtk_combo_box_new_with_model (GTK_TREE_MODEL (list_store));
   if (!writable)
     gtk_widget_set_sensitive (GTK_WIDGET (option_menu), FALSE);
-  
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (option_menu), renderer, FALSE);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (option_menu), renderer,
+                                  "text", COLUMN_STRING_TRANSLATED,
+                                  NULL);
+  g_object_set (G_OBJECT (renderer), 
+                "ellipsize-set", TRUE, 
+                "ellipsize", PANGO_ELLIPSIZE_END, 
+                "width-chars", 45, NULL);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), option_menu);
 
+  history = gm_conf_get_int (conf_key);
   while (options [cpt]) {
 
-    item = gtk_menu_item_new_with_label (options [cpt]);
-    gtk_widget_show (item);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+    gtk_list_store_append (GTK_LIST_STORE (list_store), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (list_store), &iter,
+                        COLUMN_STRING_RAW, options [cpt],
+                        COLUMN_STRING_TRANSLATED, gettext (options [cpt]),
+                        -1);
     cpt++;
   }
 
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu),
- 			       gm_conf_get_int (conf_key));
-
+  gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu), history);
   gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,
-                    (GtkAttachOptions) (GTK_FILL),
-                    (GtkAttachOptions) (GTK_FILL),
-                    0, 0);
-
+                    (GtkAttachOptions) (GTK_FILL),                
+                    (GtkAttachOptions) (GTK_FILL),                
+                    0, 0);           
+                                                                               
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && tooltip)
-    gtk_tooltips_set_tip (gpw->tips, option_menu, tooltip, NULL);
+    gtk_widget_set_tooltip_text (option_menu, tooltip);
 
-  
-  g_signal_connect (G_OBJECT (GTK_OPTION_MENU (option_menu)->menu),
-		    "deactivate", G_CALLBACK (int_option_menu_changed),
-  		    (gpointer) conf_key);
+  g_signal_connect (GTK_COMBO_BOX (option_menu), 
+		    "changed", G_CALLBACK (int_option_menu_changed),
+  		    (gpointer) conf_key);                                   
   gm_conf_notifier_add (conf_key, int_option_menu_changed_nt,
 			(gpointer) option_menu);
-
+  
   gtk_widget_show_all (table);
   
   return option_menu;
-}                                                                              
+}
 
 
 GtkWidget *
@@ -594,14 +614,18 @@ gnome_prefs_string_option_menu_new (GtkWidget *table,
 {
   GnomePrefsWindow *gpw = NULL;
   
-  GtkWidget *item = NULL;
   GtkWidget *label = NULL;                                                     
   GtkWidget *option_menu = NULL;
-  GtkWidget *menu = NULL;
+  
+  GtkListStore *list_store = NULL;
+  GtkCellRenderer *renderer = NULL;
+  GtkTreeIter iter;
+  
   gchar *conf_string = NULL;
+  gboolean writable = FALSE;
+
   int history = -1;
   int cpt = 0;
-  gboolean writable = FALSE;
 
   writable = gm_conf_is_key_writable (conf_key);
 
@@ -617,39 +641,57 @@ gnome_prefs_string_option_menu_new (GtkWidget *table,
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);                         
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
-
-  menu = gtk_menu_new ();
-  option_menu = gtk_option_menu_new ();
+  list_store = gtk_list_store_new (3, 
+                                   G_TYPE_STRING, 
+                                   G_TYPE_STRING, 
+                                   G_TYPE_BOOLEAN);
+  option_menu = gtk_combo_box_new_with_model (GTK_TREE_MODEL (list_store));
   if (!writable)
     gtk_widget_set_sensitive (GTK_WIDGET (option_menu), FALSE);
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (option_menu), renderer, FALSE);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (option_menu), renderer,
+                                  "text", COLUMN_STRING_TRANSLATED,
+                                  "sensitive", COLUMN_SENSITIVE,
+                                  NULL);
+  g_object_set (G_OBJECT (renderer), 
+                "ellipsize-set", TRUE, 
+                "ellipsize", PANGO_ELLIPSIZE_END, 
+                "width-chars", 45, NULL);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), option_menu);
 
   conf_string = gm_conf_get_string (conf_key);
-
   while (options [cpt]) {
 
-    if (conf_string != NULL)
-      if (!strcmp (conf_string, options [cpt]))
-	history = cpt;
+    if (conf_string && !strcmp (conf_string, options [cpt]))
+      history = cpt;
 
-    item = gtk_menu_item_new_with_label (options [cpt]);
-    gtk_widget_show (item);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+    gtk_list_store_append (GTK_LIST_STORE (list_store), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (list_store), &iter,
+                        COLUMN_STRING_RAW, options [cpt],
+                        COLUMN_STRING_TRANSLATED, gettext (options [cpt]),
+                        COLUMN_SENSITIVE, TRUE,
+                        -1);
     cpt++;
   }
 
   if (history == -1) {
 
-    if (options [0])
-      gm_conf_set_string (conf_key, options [0]);
-    history = 0;
+    if (conf_string && strcmp (conf_string, "")) {
+
+      gtk_list_store_append (GTK_LIST_STORE (list_store), &iter);
+      gtk_list_store_set (GTK_LIST_STORE (list_store), &iter,
+                          COLUMN_STRING_RAW, conf_string,
+                          COLUMN_STRING_TRANSLATED, gettext (conf_string),
+                          COLUMN_SENSITIVE, FALSE, 
+                          -1);
+      history = cpt;
+    }
+    else
+      history = --cpt;
   }
 
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 
- 			       history);
-
-
+  gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu), history);
   gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,
                     (GtkAttachOptions) (GTK_FILL),                
                     (GtkAttachOptions) (GTK_FILL),                
@@ -657,10 +699,10 @@ gnome_prefs_string_option_menu_new (GtkWidget *table,
                                                                                
   gpw = (GnomePrefsWindow *) g_object_get_data (G_OBJECT (table), "gpw");
   if (gpw && tooltip)
-    gtk_tooltips_set_tip (gpw->tips, option_menu, tooltip, NULL);
+    gtk_widget_set_tooltip_text (option_menu, tooltip);
 
-  g_signal_connect (G_OBJECT (GTK_OPTION_MENU (option_menu)->menu), 
-		    "deactivate", G_CALLBACK (string_option_menu_changed),
+  g_signal_connect (GTK_COMBO_BOX (option_menu), 
+		    "changed", G_CALLBACK (string_option_menu_changed),
   		    (gpointer) conf_key);                                   
   gm_conf_notifier_add (conf_key, string_option_menu_changed_nt,
 			(gpointer) option_menu);
@@ -678,23 +720,24 @@ gnome_prefs_string_option_menu_update (GtkWidget *option_menu,
 				       const gchar **options,
 				       const gchar *conf_key)
 {
-  GtkWidget *menu = NULL;
-  GtkWidget *item = NULL;
+  GtkTreeModel *model = NULL;
+  GtkTreeIter iter;
 
   gchar *conf_string = NULL;
   gchar *options_string = NULL;
   
   int history = -1;
-  int cpt = 0;                                                   
+  int cpt = 0;
 
   if (!options || !conf_key)
     return;
   
   conf_string = gm_conf_get_string (conf_key);
 
-  gtk_option_menu_remove_menu (GTK_OPTION_MENU (option_menu));
-  menu = gtk_menu_new ();
-					  
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (option_menu));
+
+  gtk_list_store_clear (GTK_LIST_STORE (model));
+
   cpt = 0;
   while (options [cpt]) {
 
@@ -702,36 +745,132 @@ gnome_prefs_string_option_menu_update (GtkWidget *option_menu,
     if (conf_string && !strcmp (options [cpt], conf_string)) 
       history = cpt;
 
-    item = gtk_menu_item_new_with_label (options_string);
-    gtk_widget_show (item);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-    
+    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
+                        COLUMN_STRING_RAW, options [cpt],
+                        COLUMN_STRING_TRANSLATED, options_string,
+                        COLUMN_SENSITIVE, TRUE, 
+                        -1);
     g_free (options_string);
-
     cpt++;
   }
 
-
   if (history == -1) {
     
-    history = 0;
+    if (conf_string && strcmp (conf_string, "")) {
 
-    if (options [0])
-      gm_conf_set_string (conf_key, options [0]);
+      gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+      gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                          COLUMN_STRING_RAW, conf_string,
+                          COLUMN_STRING_TRANSLATED, gettext (conf_string),
+                          COLUMN_SENSITIVE, FALSE, 
+                          -1);
+      history = cpt;
+    }
+    else
+      history = --cpt;
   }
 
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), history);
-
+  gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu), history);
   
-  g_signal_connect (G_OBJECT (GTK_OPTION_MENU (option_menu)->menu), 
-		    "deactivate", G_CALLBACK (string_option_menu_changed),
-  		    (gpointer) conf_key);                                   
-
   g_free (conf_string); 
 }
 
 
+void
+gnome_prefs_string_option_menu_add (GtkWidget *option_menu,
+				    const gchar *option,
+				    gboolean active)
+{
+  GtkTreeModel *model = NULL;
+  GtkTreeIter iter;
+  gchar *option_string = NULL;
+  gboolean found = FALSE;
+
+  if (!option)
+    return;
+
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (option_menu));
+  option_string = g_locale_to_utf8 (option, -1, NULL, NULL, NULL);
+
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)) {
+
+    do {
+      gchar *value_string = NULL;
+      GValue value = { 0, {{0}, {0}} };
+      gtk_tree_model_get_value (GTK_TREE_MODEL (model), &iter, 0, &value);
+      value_string = (gchar *) g_value_get_string (&value);
+      if (g_ascii_strcasecmp  (value_string, option) == 0) {
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
+                            COLUMN_SENSITIVE, TRUE,
+                            -1);
+        g_value_unset(&value);
+        found = TRUE;
+        break;
+      }
+      g_value_unset(&value);
+
+    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL (model), &iter));
+  }
+
+  if (!found) {
+    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
+                        COLUMN_STRING_RAW, option,
+                        COLUMN_STRING_TRANSLATED, option_string, 
+                        COLUMN_SENSITIVE, TRUE,
+                        -1);
+  }
+  if (active == TRUE)
+    gtk_combo_box_set_active_iter (GTK_COMBO_BOX (option_menu), &iter);
+  g_free (option_string);
+}
+
+
+
+
+void
+gnome_prefs_string_option_menu_remove (GtkWidget *option_menu,
+				       const gchar *option)
+{
+  GtkTreeModel *model = NULL;
+  GtkTreeIter iter;
+  int cpt = 0;
+  int active = 0;
+
+  if (!option)
+    return;
+
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (option_menu));
+  active = gtk_combo_box_get_active (GTK_COMBO_BOX (option_menu));
+
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)) {
+
+    do {
+      gchar *value_string = NULL;
+      GValue value = { 0, {{0}, {0}} };
+      gtk_tree_model_get_value (GTK_TREE_MODEL (model), &iter, 0, &value);
+      value_string = (gchar *) g_value_get_string (&value);
+      if (g_ascii_strcasecmp  (value_string, option) == 0) {
+
+        if (cpt == active) {
+          gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
+                              COLUMN_SENSITIVE, FALSE,
+                              -1);
+        }
+        else {
+          gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+        }
+        g_value_unset(&value);
+        break;
+      }
+      g_value_unset(&value);
+      cpt++;
+
+    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL (model), &iter));
+  }
+}
+
 GtkWidget *
 gnome_prefs_subsection_new (GtkWidget *window,
 			    GtkWidget *container,
@@ -856,10 +995,6 @@ gnome_prefs_window_new (const gchar *logo_name)
   gtk_container_add (GTK_CONTAINER (dialog_vbox), hbox);
 
 
-  /* The tooltips */
-  gpw->tips = gtk_tooltips_new ();
-
-
   /* Build the TreeView on the left */
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
