@@ -51,12 +51,6 @@
 #include "call-manager.h"
 #include "form-request-simple.h"
 
-static void
-manager_ready_in_main (Ekiga::CallManager* manager)
-{
-  manager->ready.emit ();
-}
-
 static  bool same_codec_desc (Ekiga::CodecDescription a, Ekiga::CodecDescription b)
 { 
   return (a.name == b.name && a.rate == b.rate); 
@@ -69,60 +63,37 @@ class StunDetector : public PThread
 
 public:
 
-  StunDetector (const std::string & _server, 
-                Ekiga::CallCore & _core,
-                Opal::CallManager & _manager,
-                Ekiga::Runtime & _runtime) 
+  StunDetector (const std::string & _server,
+		Opal::CallManager& _manager,
+                GAsyncQueue* _queue) 
     : PThread (1000, AutoDeleteThread), 
       server (_server),
-      core (_core),
       manager (_manager),
-      runtime (_runtime)
+      queue (_queue)
   {
+    g_async_queue_ref (queue);
     this->Resume ();
   };
 
   ~StunDetector ()
   {
-    if (!nat_error.empty ()) {
-      while (!core.errors.handle_request (nat_error)) {
-        PThread::Current ()->Sleep (100);
-      }
-    }
+    g_async_queue_unref (queue);
   }
   
   void Main () 
   {
-    PSTUNClient::NatTypes type = manager.SetSTUNServer (server);
-    if (type == PSTUNClient::SymmetricNat 
-        || type == PSTUNClient::BlockedNat 
-        || type == PSTUNClient::PartialBlockedNat) {
+    PSTUNClient::NatTypes* result = NULL;
 
-      nat_error =  _("Ekiga did not manage to configure your network settings automatically. You can"
-                     " still use it, but you need to configure your network settings manually.\n\n"
-                     "Please see http://wiki.ekiga.org/index.php/Enable_port_forwarding_manually for"
-                     " instructions");
+    result = (PSTUNClient::NatTypes*)g_malloc0 (sizeof (PSTUNClient::NatTypes));
+    *result = manager.SetSTUNServer (server);
 
-
-    }
-    else {
-
-      for (Ekiga::CallManager::iterator iter = manager.begin ();
-           iter != manager.end ();
-           iter++) 
-        (*iter)->set_listen_port ((*iter)->get_listen_interface ().port);
-    }
-
-    runtime.run_in_main (sigc::bind (sigc::ptr_fun (manager_ready_in_main),
-				     &manager));
+    g_async_queue_push (queue, result);
   };
 
 private:
   const std::string server;
-  Ekiga::CallCore & core;
   Opal::CallManager & manager;
-  Ekiga::Runtime & runtime;
-  std::string nat_error;
+  GAsyncQueue* queue;
 };
 
 
@@ -136,7 +107,8 @@ CallManager::CallManager (Ekiga::ServiceCore & _core)
 {
   /* Initialise the endpoint paramaters */
   PIPSocket::SetDefaultIpAddressFamilyV4();
-  autoStartTransmitVideo = autoStartReceiveVideo = true;
+  SetAutoStartTransmitVideo (true);
+  SetAutoStartReceiveVideo  (true);
   SetUDPPorts (5000, 5100);
   SetTCPPorts (30000, 30100);
   SetRtpIpPorts (5000, 5100);
@@ -171,19 +143,34 @@ CallManager::CallManager (Ekiga::ServiceCore & _core)
 
   //
   call_core = dynamic_cast<Ekiga::CallCore *> (core.get ("call-core"));
+
+
+  // used to communicate with the StunDetector
+#if GLIB_CHECK_VERSION(2,16,0)
+  queue = g_async_queue_new_full (g_free);
+#else
+  queue = g_async_queue_new ();
+#endif
+
+  PInterfaceMonitor::GetInstance().SetRefreshInterval (15000);
 }
 
 
 CallManager::~CallManager ()
 {
   ClearAllCalls (OpalConnection::EndedByLocalUser, false);
+
+  g_async_queue_unref (queue);
 }
 
 
 void CallManager::start ()
 {
   // Ready
-  new StunDetector ("stun.voxgratia.org", *call_core, *this, runtime);
+  new StunDetector (stun_server, *this, queue);
+
+  patience = 20;
+  runtime.run_in_main (sigc::mem_fun (this, &CallManager::HandleSTUNResult), 1);
 }
 
 
@@ -260,13 +247,15 @@ void CallManager::set_maximum_jitter (unsigned max_val)
       PSafePtr<OpalRTPConnection> connection = PSafePtrCast<OpalConnection, OpalRTPConnection> (call->GetConnection (i));
       if (connection) {
 
-        RTP_Session *session = 
-          connection->GetSession (OpalMediaFormat::DefaultAudioSessionID);
+        OpalMediaStreamPtr stream = connection->GetMediaStream (OpalMediaType::Audio (), false);
+        if (stream != NULL) {
 
-        if (session != NULL) {
+          RTP_Session *session = connection->GetSession (stream->GetSessionID ());
+          if (session != NULL) {
 
-          unsigned units = session->GetJitterTimeUnits ();
-          session->SetJitterBufferSize (20 * units, max_val * units, units);
+            unsigned units = session->GetJitterTimeUnits ();
+            session->SetJitterBufferSize (20 * units, max_val * units, units);
+          }
         }
       }
     }
@@ -432,10 +421,6 @@ void CallManager::set_codecs (Ekiga::CodecList & _codecs)
   // Update the OpalManager
   SetMediaFormatMask (mask);
   SetMediaFormatOrder (order);
-
-  // Output Mask
-  PTRACE (4, "OPAL\tMediaFormat Mask set to " << setfill (',') << mask << setfill (' '));
-  PTRACE (4, "OPAL\tMediaFormat Order set to " << setfill (',') << order << setfill (' '));
 }
 
 void CallManager::set_forward_on_no_answer (bool enabled)
@@ -502,6 +487,13 @@ void CallManager::get_tcp_ports (unsigned & min_port,
 }
 
 
+void CallManager::set_stun_server (const std::string & server)
+{
+  stun_server = server;
+  start ();
+}
+
+
 bool CallManager::dial (const std::string & uri)
 {
   for (CallManager::iterator iter = begin ();
@@ -523,7 +515,7 @@ void CallManager::set_video_options (const CallManager::VideoOptions & options)
   for (int i = 0 ; i < media_formats_list.GetSize () ; i++) {
 
     OpalMediaFormat media_format = media_formats_list [i];
-    if (media_format.GetDefaultSessionID () == OpalMediaFormat::DefaultVideoSessionID) {
+    if (media_format.GetMediaType() == OpalMediaType::Video ()) {
 
       media_format.SetOptionInteger (OpalVideoFormat::FrameWidthOption (), 
                                      Ekiga::VideoSizes [options.size].width);  
@@ -578,12 +570,10 @@ void CallManager::set_video_options (const CallManager::VideoOptions & options)
          i < 2;
          i++) {
 
-      PSafePtr<OpalConnection> connection = call->GetConnection (i);
+      PSafePtr<OpalRTPConnection> connection = PSafePtrCast<OpalConnection, OpalRTPConnection> (call->GetConnection (i));
       if (connection) {
 
-        OpalMediaStream *stream = 
-          connection->GetMediaStream (OpalMediaFormat::DefaultVideoSessionID, false); 
-
+        OpalMediaStreamPtr stream = connection->GetMediaStream (OpalMediaType::Video (), false);
         if (stream != NULL) {
 
           OpalMediaFormat mediaFormat = stream->GetMediaFormat ();
@@ -608,7 +598,7 @@ void CallManager::get_video_options (CallManager::VideoOptions & options) const
   for (int i = 0 ; i < media_formats_list.GetSize () ; i++) {
 
     OpalMediaFormat media_format = media_formats_list [i];
-    if (media_format.GetDefaultSessionID () == OpalMediaFormat::DefaultVideoSessionID) {
+    if (media_format.GetMediaType () == OpalMediaType::Video ()) {
 
       int j = 0;
       for (j = 0; j < NB_VIDEO_SIZES; j++) {
@@ -671,25 +661,13 @@ CallManager::OnOpenMediaStream (OpalConnection & connection,
 }
 
 
-void 
-CallManager::OnMWIReceived (const PString & _acc,
-                            MessageWaitingType /*type*/,
-                            const PString & _msgs)
-{
-  std::string account = (const char *) _acc;
-  std::string summary = (const char *) _msgs;
-
-  runtime.run_in_main (sigc::bind (mwi_event.make_slot (), account, summary));
-}
-
-
-
 void CallManager::GetAllowedFormats (OpalMediaFormatList & full_list)
 {
   OpalMediaFormatList list = OpalTranscoder::GetPossibleFormats (pcssEP->GetMediaFormats ());
   std::list<std::string> black_list;
    
   black_list.push_back ("GSM-AMR");
+  black_list.push_back ("Linear-16-Stereo-48kHz");
   black_list.push_back ("LPC-10");
   black_list.push_back ("SpeexIETFNarrow-11k");
   black_list.push_back ("SpeexIETFNarrow-15k");
@@ -710,3 +688,53 @@ void CallManager::GetAllowedFormats (OpalMediaFormatList & full_list)
   }
 }
 
+void
+CallManager::HandleSTUNResult ()
+{
+  PSTUNClient::NatTypes* result = NULL;
+
+  result = (PSTUNClient::NatTypes*)g_async_queue_try_pop (queue);
+
+  if (result != NULL || patience == 0) {
+
+    if (patience == 0
+	|| *result == PSTUNClient::SymmetricNat
+	|| *result == PSTUNClient::BlockedNat
+	|| *result == PSTUNClient::PartialBlockedNat) {
+
+      ReportSTUNError (_("Ekiga did not manage to configure your network settings automatically. You can"
+			 " still use it, but you need to configure your network settings manually.\n\n"
+			 "Please see http://wiki.ekiga.org/index.php/Enable_port_forwarding_manually for"
+			 " instructions"));
+    } else {
+
+    for (Ekiga::CallManager::iterator iter = begin ();
+	 iter != end ();
+	 ++iter) 
+      (*iter)->set_listen_port ((*iter)->get_listen_interface ().port);
+    }
+
+    ready.emit ();
+
+    if (result != NULL)
+      g_free (result);
+
+  } else {
+
+      patience--;
+      runtime.run_in_main (sigc::mem_fun (this,
+					  &CallManager::HandleSTUNResult),
+			   1);
+  }
+}
+
+void
+CallManager::ReportSTUNError (const std::string error)
+{
+  // notice we're in for an infinite loop if nobody ever reports to the user!
+  if ( !call_core->errors.handle_request (error))
+    runtime.run_in_main (sigc::bind (sigc::mem_fun (this,
+						    &CallManager::ReportSTUNError),
+				     error),
+			 10);
+}
