@@ -349,8 +349,8 @@ void Opal::Sip::EndPoint::unfetch (const std::string uri)
 void Opal::Sip::EndPoint::publish (const Ekiga::PersonalDetails & details)
 {
   std::string hostname = (const char *) PIPSocket::GetHostName ();
-  std::string short_status = ((Ekiga::PersonalDetails &) (details)).get_short_status ();
-  std::string long_status = ((Ekiga::PersonalDetails &) (details)).get_long_status ();
+  std::string presence = ((Ekiga::PersonalDetails &) (details)).get_presence ();
+  std::string status = ((Ekiga::PersonalDetails &) (details)).get_status ();
 
   for (std::list<std::string>::iterator it = aors.begin ();
        it != aors.end ();
@@ -370,10 +370,10 @@ void Opal::Sip::EndPoint::publish (const Ekiga::PersonalDetails & details)
     data += "\">\r\n";
 
     data += "<note>";
-    data += short_status.c_str ();
-    if (!long_status.empty ()) {
+    data += presence.c_str ();
+    if (!status.empty ()) {
       data += " - ";
-      data += long_status.c_str ();
+      data += status.c_str ();
     }
     data += "</note>\r\n";
 
@@ -390,7 +390,7 @@ void Opal::Sip::EndPoint::publish (const Ekiga::PersonalDetails & details)
     data += "</tuple>\r\n";
     data += "</presence>\r\n";
 
-    Publish (to, data, 300); // TODO: allow to change the 500 
+    Publish (to, data, 500); // TODO: allow to change the 500 
   }
 }
 
@@ -1062,15 +1062,7 @@ SIPURL Opal::Sip::EndPoint::GetRegisteredPartyName (const SIPURL & host)
   /* As a last resort, ie not registered to host, no default account or
    * dialog with a local peer, then use the local address 
    */
-  local_address = GetListeners()[0].GetLocalAddress();
-
-  PINDEX j = local_address.Find ('$');
-  if (j != P_MAX_INDEX)
-    local_address = local_address.Mid (j+1);
-  SIPURL myself = 
-    SIPURL ("\"" + GetDefaultDisplayName () + "\" <" + PString ("sip:") + GetDefaultLocalPartyName() + "@" + local_address + ";transport=udp>");
-
-  return myself;
+  return GetDefaultRegisteredPartyName ();
 }
 
 
@@ -1083,38 +1075,50 @@ Opal::Sip::EndPoint::OnPresenceInfoReceived (const PString & user,
   PCaselessString b = basic;
   PCaselessString s = note;
 
-  std::string status;
   std::string presence = "unknown";
+  std::string status;
 
-  if (b.Find ("Closed") != P_MAX_INDEX)
-    presence = "offline";
-  else
-    presence = "online";
+  if (!basic.IsEmpty () && !note.IsEmpty ()) {
 
-  if (s.Find ("Away") != P_MAX_INDEX)
-    presence = "away";
-  else if (s.Find ("On the phone") != P_MAX_INDEX
-           || s.Find ("Ringing") != P_MAX_INDEX) 
-    presence = "inacall";
-  else if (s.Find ("dnd") != P_MAX_INDEX
-           || s.Find ("Do Not Disturb") != P_MAX_INDEX) 
-    presence = "dnd";
-
-  else if (s.Find ("Free For Chat") != P_MAX_INDEX) 
-    presence = "freeforchat";
-
-  if ((j = s.Find (" - ")) != P_MAX_INDEX)
-    status = (const char *) note.Mid (j + 3);
+    if (b.Find ("Open") != P_MAX_INDEX)
+      presence = "online";
+    else
+      presence = "offline";
+  
+    if (s.Find ("Away") != P_MAX_INDEX)
+      presence = "away";
+    else if (s.Find ("On the phone") != P_MAX_INDEX
+             || s.Find ("Ringing") != P_MAX_INDEX) 
+      presence = "inacall";
+    else if (s.Find ("dnd") != P_MAX_INDEX
+             || s.Find ("Do Not Disturb") != P_MAX_INDEX) 
+      presence = "dnd";
+  
+    else if (s.Find ("Free For Chat") != P_MAX_INDEX) 
+      presence = "freeforchat";
+  
+    if ((j = s.Find (" - ")) != P_MAX_INDEX)
+      status = (const char *) note.Mid (j + 3);
+  }
 
   SIPURL sip_uri = SIPURL (user);
   sip_uri.Sanitise (SIPURL::ExternalURI);
   std::string _uri = sip_uri.AsString ();
+  std::string old_presence = uri_presences[_uri].first;
+  std::string old_status = uri_presences[_uri].second;
+  
+  // If first notification, and no information, then we are offline
+  if (presence == "unknown" && old_presence.empty ())
+    presence = "offline";
 
-  /**
-   * TODO
-   * Wouldn't it be convenient to emit the signal and have the presence core listen to it ?
-   */
-  runtime.run_in_main (sigc::bind (sigc::ptr_fun (presence_status_in_main), this, _uri, presence, status));
+  // If presence change, then signal it to the various components
+  // If presence is unknown (notification with empty body), and it is not the 
+  // first notification, and we can conclude it is a ping back from the server 
+  // to indicate the presence status did not change, hence we do nothing.
+  if (presence != "unknown" && (old_presence != presence || old_status != status)) {
+    uri_presences[_uri] = std::pair<std::string, std::string> (presence, status);
+    runtime.run_in_main (sigc::bind (sigc::ptr_fun (presence_status_in_main), this, _uri, presence, status));
+  }
 }
 
 
