@@ -42,6 +42,7 @@
 #include "account.h"
 #include "bank.h"
 #include "account-core.h"
+#include "audiooutput-core.h"
 
 #include "callbacks.h"
 #include "misc.h"
@@ -49,8 +50,6 @@
 #include "preferences.h"
 
 #include "manager.h"
-#include "sip-endpoint.h"
-#include "h323-endpoint.h"
 #include "ekiga.h"
 
 #include "gmconf.h"
@@ -151,11 +150,79 @@ gm_aw_get_aw (GtkWidget *accounts_window)
 }
 
 
+bool
+gm_accounts_window_update_account_state (GtkWidget *accounts_window,
+					 gboolean refreshing,
+                                         const Ekiga::Account & _account,
+					 const gchar *status,
+					 const gchar *voicemails)
+{
+  GtkTreeModel *model = NULL;
+
+  GtkTreeIter iter;
+
+  Ekiga::Account *account = NULL;
+  GmAccountsWindow *aw = NULL;
+
+  gchar *error = NULL;
+  gchar *mwi = NULL;
+
+  bool status_modified = false;
+  bool mwi_modified = false;
+
+  g_return_val_if_fail (accounts_window != NULL, false);
+
+  aw = gm_aw_get_aw (accounts_window);
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (aw->accounts_list));
+
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)){
+
+    do {
+
+      gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+			  COLUMN_ACCOUNT, &account,
+			  -1);
+
+      if (account == &_account) {
+
+        gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+                            COLUMN_ACCOUNT_ERROR_MESSAGE, &error,
+                            COLUMN_ACCOUNT_VOICEMAILS, &mwi,
+                            -1);
+
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			    COLUMN_ACCOUNT_STATE, refreshing, -1);
+	if (status) {
+
+	  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			      COLUMN_ACCOUNT_ERROR_MESSAGE, status, -1);
+          status_modified = (error == NULL) || strcmp (status, error);
+        }
+	if (voicemails) {
+
+	  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			      COLUMN_ACCOUNT_VOICEMAILS, voicemails, -1);
+          mwi_modified = (mwi == NULL) || strcmp (voicemails, mwi);
+        }
+      
+        g_free (error);
+        g_free (mwi);
+      }
+
+    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter));
+  }
+
+  return (status_modified || mwi_modified);
+}
+
+
 /* Engine callbacks */
-static void on_registration_event (const Ekiga::Account & account,
-                                   Ekiga::AccountCore::RegistrationState state,
-                                   std::string info,
-                                   gpointer window)
+static void 
+on_registration_event (const Ekiga::Account & account,
+                       Ekiga::AccountCore::RegistrationState state,
+                       std::string info,
+                       gpointer window)
 {
   bool is_processing = false;
   std::string status;
@@ -188,8 +255,33 @@ static void on_registration_event (const Ekiga::Account & account,
     break;
   }
 
-  gm_accounts_window_update_account_state (GTK_WIDGET (window), is_processing, 
-                                           account, status.c_str (), NULL); 
+  gm_accounts_window_update_account_state (GTK_WIDGET (window), is_processing, account, status.c_str (), NULL); 
+}
+
+
+static void 
+on_mwi_event (const Ekiga::Account & account,
+              std::string mwi,
+              gpointer self)
+{
+  GmAccountsWindow *aw = NULL;
+
+  aw = gm_aw_get_aw (GTK_WIDGET (self));
+  Ekiga::AudioOutputCore *audiooutput_core = dynamic_cast<Ekiga::AudioOutputCore *> (aw->core.get ("audiooutput-core"));
+
+  if (gm_accounts_window_update_account_state (GTK_WIDGET (self), false, account, NULL, mwi.c_str ())) {
+
+    std::string::size_type loc = mwi.find ("/", 0);
+    if (loc != std::string::npos) {
+
+      std::stringstream new_messages;
+      int i;
+      new_messages << mwi.substr (0, loc);
+      new_messages >> i;
+      if (i > 0)
+        audiooutput_core->play_event ("new_voicemail_sound");
+    }
+  }
 }
 
 
@@ -314,6 +406,8 @@ account_toggled_cb (G_GNUC_UNUSED GtkCellRendererToggle *cell,
   GtkTreeSelection *selection = NULL;
   GtkTreeIter iter;
 
+  gboolean fixed = false;
+
   Ekiga::Account *account = NULL;
 
   aw = gm_aw_get_aw (GTK_WIDGET (data));
@@ -327,11 +421,19 @@ account_toggled_cb (G_GNUC_UNUSED GtkCellRendererToggle *cell,
   if (gtk_tree_model_get_iter (model, &iter, path)) {
 
     gtk_tree_model_get (model, &iter,
+                        COLUMN_ACCOUNT_ENABLED, &fixed,
                         COLUMN_ACCOUNT, &account,
                         -1);
 
-    bool enabled = account->is_enabled ();
-    enabled ? account->disable () : account->enable ();
+    if (fixed)
+      account->disable ();
+    else 
+      account->enable ();
+
+    gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                        COLUMN_ACCOUNT_ENABLED, fixed^1,
+                        COLUMN_ACCOUNT_ERROR_MESSAGE, "",
+                        -1);
   }
 
   gtk_tree_path_free (path);
@@ -501,13 +603,15 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
   GtkWidget *window = NULL;
 
   GtkWidget *menu_bar = NULL;
+  GtkWidget *menu_item = NULL;
+  GtkWidget *menu = NULL;
+  GtkWidget *item = NULL;
   GtkWidget *event_box = NULL;
   GtkWidget *scroll_window = NULL;
 
   GtkWidget *frame = NULL;
   GtkWidget *hbox = NULL;
 
-  GtkTreeModel *model = NULL;
   GtkCellRenderer *renderer = NULL;
   GtkListStore *list_store = NULL;
   GtkTreeViewColumn *column = NULL;
@@ -546,12 +650,18 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
   gtk_window_add_accel_group (GTK_WINDOW (window), aw->accel);
   g_object_unref (aw->accel);
 
-  aw->menu_item_core = 
-    gtk_menu_item_new_with_mnemonic (_("Account"));
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar),
-                         aw->menu_item_core);
+  aw->menu_item_core = gtk_menu_item_new_with_mnemonic (_("_Accounts"));
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), aw->menu_item_core);
   g_object_ref (aw->menu_item_core);
 
+  menu_item = gtk_menu_item_new_with_mnemonic (_("_Help"));
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);
+
+  menu = gtk_menu_new ();
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
+  item = gtk_image_menu_item_new_from_stock (GTK_STOCK_HELP, NULL);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+  g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (help_cb), NULL);
 
   /* The accounts list store */
   list_store = gtk_list_store_new (COLUMN_ACCOUNT_NUMBER,
@@ -563,8 +673,7 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
 				   G_TYPE_STRING,  /* Error Message */  
 				   G_TYPE_INT);    /* State */
 
-  aw->accounts_list = 
-    gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
+  aw->accounts_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
   g_object_unref (list_store);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (aw->accounts_list), TRUE);
   gtk_tree_view_set_reorderable (GTK_TREE_VIEW (aw->accounts_list), TRUE);
@@ -573,6 +682,11 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
   atk_object_set_name (aobj, _("Accounts"));
 
   renderer = gtk_cell_renderer_toggle_new ();
+  /* Translators:
+   * The following string "A" means "activated" or "active". It's a column
+   * description in the list of configured accounts, it shows if an account
+   * is activated or not (a status the user can choose).
+   */
   column = gtk_tree_view_column_new_with_attributes (_("A"),
 						     renderer,
 						     "active", 
@@ -605,15 +719,9 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
     gtk_tree_view_column_set_sort_column_id (column, i);
   }
 
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (aw->accounts_list));
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
-                                        COLUMN_ACCOUNT_ENABLED,
-                                        GTK_SORT_DESCENDING);
-
   g_signal_connect (G_OBJECT (aw->accounts_list), "event_after",
 		    G_CALLBACK (account_clicked_cb), window);
   
-
   /* The scrolled window with the accounts list store */
   scroll_window = gtk_scrolled_window_new (FALSE, FALSE);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_window), 
@@ -622,7 +730,7 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
 
   event_box = gtk_event_box_new ();
   hbox = gtk_hbox_new (FALSE, 4);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
   gtk_container_add (GTK_CONTAINER (event_box), hbox);
 
   frame = gtk_frame_new (NULL);
@@ -662,52 +770,7 @@ gm_accounts_window_new (Ekiga::ServiceCore &core)
   account_core->account_removed.connect (sigc::bind (sigc::ptr_fun (on_account_removed), window));
   account_core->questions.add_handler (sigc::bind (sigc::ptr_fun (on_handle_questions), (gpointer) window));
   account_core->registration_event.connect (sigc::bind (sigc::ptr_fun (on_registration_event), (gpointer) window));
+  account_core->mwi_event.connect (sigc::bind (sigc::ptr_fun (on_mwi_event), (gpointer) window));
   
   return window;
 }
-
-
-void
-gm_accounts_window_update_account_state (GtkWidget *accounts_window,
-					 gboolean refreshing,
-                                         const Ekiga::Account & _account,
-					 const gchar *status,
-					 const gchar *voicemails)
-{
-  GtkTreeModel *model = NULL;
-
-  GtkTreeIter iter;
-
-  Ekiga::Account *account = NULL;
-  GmAccountsWindow *aw = NULL;
-
-  g_return_if_fail (accounts_window != NULL);
-
-  aw = gm_aw_get_aw (accounts_window);
-
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (aw->accounts_list));
-
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)){
-
-    do {
-
-      gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-			  COLUMN_ACCOUNT, &account,
-			  -1);
-
-      if (account == &_account) {
-
-	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			    COLUMN_ACCOUNT_STATE, refreshing, -1);
-	if (status)
-	  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			      COLUMN_ACCOUNT_ERROR_MESSAGE, status, -1);
-	if (voicemails)
-	  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			      COLUMN_ACCOUNT_VOICEMAILS, voicemails, -1);
-      }
-
-    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter));
-  }
-}
-
